
/**
  ******************************************************************************
  * @file   : main.c
  * @author : Softweb Solutions An Avnet Company
  * @modify : 22-Feb-2021
  * @brief  : Firmware part for C_STM32L4 v3.9
  ******************************************************************************
*/
 
/**
 * Hope you have installed the STM32L4_SDK required packages and software as guided on SDK documentation. 
 * Import required packages
 */
#include <stdio.h>
#include "cJSON.c"
#include "azure_config.h"
#include "iothub_client_ll.h"
#include "STM32CubeInterface.h"
#include "AzureClientApplication.h"

IOTHUB_CLIENT_LL_HANDLE iotHubClientHandle;

/**
 * ## Prerequisite configuration to run this sample code
 * UniqueId          :: Its device ID which register on IotConnect platform and also its status has Active and Acquired
 * Sid               :: It need to get from the IoTConnect platform "Settings->Key Vault". 
 * ConnectionString  :: It gets from the IoTConnect platform Device Detail page.
 * sdkOptions        :: It helps to define other configurations but keep it NULL for now
 * TIME_FORMAT       :: It is time format "YYYY-MM-DDTHH:MM:SS.FFFZ" eg. 2020-01-21T22:57:59.968Z
 */
char *UniqueId = "Your_Device_ID"; 
char *Sid = "Your_SDK_ID"; 	 
char *ConnectionString = "Device_Connection_String";
char *sensors_data = NULL;
static const char TIME_FORMAT[]="%04u-%02u-%02uT%02u:%02u:%02u.%03uZ";

void ReadSensordata(void);

/*! Object  - will Get RTC time
 *  \param  - need a pointer buffer with size
 *  \return - None
 */
static void FormatTime(char* buf, size_t bufSize) {
	RTC_DateTypeDef sdatestructureget;
	RTC_TimeTypeDef stimestructureget;

	/* Get the RTC current Time */
	HAL_RTC_GetTime(&TargetBoardFeatures.RtcHandle, &stimestructureget, RTC_FORMAT_BIN);
	/* Get the RTC current Date */
	HAL_RTC_GetDate(&TargetBoardFeatures.RtcHandle, &sdatestructureget, RTC_FORMAT_BIN);
	sprintf_s(buf, bufSize, TIME_FORMAT,
		2000 + sdatestructureget.Year,
				sdatestructureget.Month,
				sdatestructureget.Date,
				stimestructureget.Hours,
				stimestructureget.Minutes,
				stimestructureget.Seconds,
		1000 * stimestructureget.SubSeconds / stimestructureget.SecondFraction
	);
}

/*! Object  - application main function
 *  \param  - None
 *  \return - None
 */
void main(void)
{
	//platform application
	AzureClientApplication();

	char *SdkOptions = NULL;
	cJSON *sdk_object = cJSON_CreateObject();
	SdkOptions = cJSON_Print(sdk_object);

	/**
	 * Init. you have to pass Sid, ConnectionString, SdkOptions, DeviceCallback, TwinUpdateCallback  All variables are defined above.
	 * Note : optional Params and sdkOptions.
	 * Usage   : To Initialize SDK and Device connection
	 * Input   : Sid, ConnectionString, SdkOptions as explained above
	 * Output  : Callback methods for command and twin properties
	 */
	Init(Sid, ConnectionString, SdkOptions, DeviceCallback, TwinUpdateCallback);
	HAL_Delay(5000);
  
  	/**
	 * Type    : Public Method "getAllTwins()"
	 * Usage   : Send request to get all the twin properies Desired and Reported
	 * Input   : 
	 * Output  : 
	 */
  	//GetAllTwins();

  	//sending data to cloud only for 30 times
  	int a=0;
  	while(a<30){
	  	a++;
	  	ReadSensordata();

		/**
		 * Type    : Public Method "SendSensorData()"
		 * Usage   : To publish the D2C data
		 * Input   : Predefined data object
		 * Output  :
		 */
	  	SendSensorData(sensors_data);
	  	IoTHubClient_LL_DoWork(iotHubClientHandle);
	 	//  send every 30 seconds or so...
		HAL_Delay(30*1000);
	}

	/**
	 *  Type    : Public Method "Dispose()"
	 *  Usage   : Disconnect the device from cloud
	 *  Input   :
	 *  Output  :
	 */
	Dispose();

}

/*! Object  - need to read and setup all sensor(Attribute) data in JSON format
 *  \param  - None
 *  \return - None
 */
void ReadSensordata(void){

	char tsStrBuf[24 + 1]; 	// time format : YYYY-MM-DDTHH:MM:SS.FFFZ eg. 2020-01-21T22:57:59.968Z
	FormatTime(tsStrBuf, sizeof(tsStrBuf));
    cJSON *obj, *obj2, *root_array, *data, *data2, *Gy;

	// For Non Gateway Device
    root_array = cJSON_CreateArray();
    obj = cJSON_CreateObject();
    cJSON_AddItemToArray(root_array,obj);
    cJSON_AddStringToObject(obj, "uniqueId",UniqueId);
    cJSON_AddStringToObject(obj, "time",tsStrBuf);
    cJSON_AddItemToObject(obj, "data", data=cJSON_CreateObject());
    cJSON_AddNumberToObject(data, "Temperature",56);
    cJSON_AddStringToObject(data, "String", "P11");
    cJSON_AddItemToObject(data, "Gyroscope", Gy=cJSON_CreateObject());
    cJSON_AddNumberToObject(Gy, "x",44);
    cJSON_AddStringToObject(Gy, "y", "P22");
    cJSON_AddNumberToObject(Gy, "z",55);
		
	/**
	 * Non Gateway device input data format Example:
	 *	String data = [{
	 *		"uniqueId": "<< Device UniqueId >>",
	 *		"time" : "<< date >>",
	 *		"data": {}
	 *	}];
	 */

	// For Gateway Device
    /* 
	 * root_array = cJSON_CreateArray();
     * obj = cJSON_CreateObject();
     * cJSON_AddItemToArray(root_array,obj);
     * cJSON_AddStringToObject(obj, "uniqueId",UniqueId);
     * cJSON_AddStringToObject(obj, "time",tsStrBuf);
     * cJSON_AddItemToObject(obj, "data", data=cJSON_CreateObject());
     * cJSON_AddNumberToObject(data, "Temperature",56);
     * cJSON_AddStringToObject(data, "String", "P11");
	 * 	obj2 = cJSON_CreateObject();
	 * 	cJSON_AddItemToArray(root_array,obj2);
	 * 	cJSON_AddStringToObject(obj2, "uniqueId","ChildId");
	 * 	cJSON_AddStringToObject(obj2, "time",tsStrBuf);
	 * 	cJSON_AddItemToObject(obj2, "data", data2=cJSON_CreateObject());
	 * 	cJSON_AddNumberToObject(data2, "Temperature",56);
	 * 	cJSON_AddStringToObject(data2, "colour", "red");
 	 */

	/**
	 * Gateway device input data format Example:
	 * For Gateway and multiple child device 
	 *	String data = [{
	 *		"uniqueId": "<< Gateway Device UniqueId >>", // It should be must first object of the array
	 *		"time": "<< date >>",
	 *		"data": {}
	 *	},
	 *	{
	 *		"uniqueId":"<< Child DeviceId >>", //Child device
	 *		"time": "<< date >>",
	 *		"data": {}
	 *	}]
	 *
	 */
      
	sensors_data = cJSON_Print(root_array);
    cJSON_Delete(root_array);
}

/*! Object  - Will give you Device callback payload from C2D with ACK from D2C
 *  \param  - payload msg from SDK
 *  \return - None
 */
void DeviceCallback(char *payload){

	int Status=0,msgType=0,CMD = 0;
	char *value=NULL, *cmd_ackID=NULL, *cmd_UNI = "";
	cJSON *root_value, *in_url, *sub_value, *Ack_Json;
	root_value = cJSON_Parse(payload);
    if (root_value == NULL)  {
        const char *error_ptr = cJSON_GetErrorPtr();
        if (error_ptr != NULL){
        	AZURE_PRINTF(stderr, "\r\n\tError before: %s\n", error_ptr);
        }
    }

    sub_value = cJSON_GetObjectItem(root_value,"ackId");
	if(cJSON_IsString(sub_value))
		cmd_ackID = sub_value->valuestring;
	else AZURE_PRINTF("\r\n\tNot getting any ackId in CMD\n");

	sub_value = cJSON_GetObjectItem(root_value,"cmdType");
	if(cJSON_IsString(sub_value))
		value = sub_value->valuestring;
	else AZURE_PRINTF("\r\n\tNot getting any cmdType in CMD\n");

	if(strcmp(value,"0x16") ==0){
		sub_value = cJSON_GetObjectItem(root_value,"command");
		CMD = sub_value->valueint;

		if(CMD == 1){
		AZURE_PRINTF("\r\n\t ** Device Connected ** \r\n");
		}else if(CMD == 0)
			AZURE_PRINTF("\r\n\t ** Device Disconnected ** \r\n");

		return ;
	}
	else if(strcmp(value,"0x01") ==0){
		Status = 6; msgType = 5;
	}
	else if(strcmp(value,"0x02") ==0){
		Status = 7; msgType = 11;
		sub_value = cJSON_GetObjectItem(root_value,"urls");
		if(cJSON_IsArray(sub_value)){
			in_url = cJSON_GetArrayItem(sub_value, 0);
			sub_value = cJSON_GetObjectItem(in_url, "uniqueId");
			if(cJSON_IsString(sub_value))
				cmd_UNI = sub_value->valuestring;
		}
	}

	Ack_Json=cJSON_CreateObject();
	cJSON_AddStringToObject(Ack_Json, "ackId", cmd_ackID);
	cJSON_AddStringToObject(Ack_Json, "msg", "");
	cJSON_AddStringToObject(Ack_Json, "childId", cmd_UNI);
	cJSON_AddNumberToObject(Ack_Json, "st", Status);
    char *Ack_Json_Data = cJSON_Print(Ack_Json);
	char AckTime[24 + 1]; 	// time format : YYYY-MM-DDTHH:MM:SS.FFFZ eg. 2020-01-21T22:57:59.968Z
	FormatTime(AckTime, sizeof(AckTime));
    
    /**
    *  Type    : Public Method "sendAck()"
    *  Usage   : Send firmware command received acknowledgement to cloud
    *  - status Type
	*	st = 6; // Device command Ack status 
    *    st = 7; // firmware OTA command Ack status 
    *    st = 4; // Failed Ack
    *  - Message Type
	*	msgType = 5; // for "0x01" device command 
    *    msgType = 11; // for "0x02" Firmware command
    */
	if(cJSON_IsObject(Ack_Json))
    	SendAck(Ack_Json_Data, AckTime, msgType);
}

/**
 * Type    : Callback Function "TwinUpdateCallback()"
 * Usage   : Manage twin properties as per business logic to update the twin reported property
 * Input   : 
 * Output  : Receive twin Desired and twin Reported properties
 */
void TwinUpdateCallback(char *payload){

	/* Here we will get device twin properties payload from cloud to device */
	AZURE_PRINTF("\r\n *** Twin MSG >>Payload : %s\r\n", payload);
}
